﻿using System;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Scrum.Web.Controllers;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Moq;
using System.Web;
using System.Linq.Expressions;
using Scrum.Web.Models;
using Scrum.Web;
using System.Web.Security;


namespace Scrum.Tests.Controllers
{
    [TestClass]
    public class AccountControllerTest : BaseControllerTest
    {
        private AccountController _controller;
        
        [TestInitialize]
        public void TestInitialize()
        {
            _controller = new AccountController(_mockRepository.Object, _mockScrumMembershipProvider.Object, _mockScrumRoleProvider.Object);
            var mockedList = _controller.SetFakeControllerContextAndReturnMocksList("~/Account/");

            //_controller.GetMockedObjectUsing<HttpContextBase>(mockedList)
            //    .SetupGet(ctx => ctx.User.Identity.Name).Returns(us);

            _mockRepository
                .Setup(ctx => ctx.Single<User>(It.IsAny<Expression<Func<User, bool>>>()))
                .Returns(GetUser_Admin());

            _mockScrumMembershipProvider.Setup(ctx => ctx.GetUser()).Returns(GetMembershipUser_Admin);
        }

        

        [TestMethod]
        public void ManageIsNotNull()
        {
            // Actuar
            ViewResult result = _controller.Manage((Scrum.Web.Controllers.AccountController.ManageMessageId?)null) as ViewResult;

            // Declarar
            Assert.IsNotNull(result);
        }

        [TestMethod]
        public void ManageMapCorrectExistUser()
        {
            // Actuar
            ViewResult result = _controller.Manage((Scrum.Web.Controllers.AccountController.ManageMessageId?)null) as ViewResult;

            // Declarar
            var model = result.Model as PersonalDataEditModel;
            var user = GetUser_Admin();
            Assert.IsNotNull(model);
            Assert.AreEqual(model.Name, user.Name);
            Assert.AreEqual(model.Email, user.Email);
            Assert.AreEqual(model.NewPassword, null);
            Assert.AreEqual(model.ConfirmPassword, null);
            Assert.AreEqual(model.UserId, user.UserId);
        }

        [TestMethod]
        public void ManageChangePersonalDataOK()
        {
            var personalData = new Mock<PersonalDataEditModel>();
            // Actuar
            ViewResult result = _controller.Manage(personalData.Object) as ViewResult;

            // Declarar
            
        }
    }
}
