﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Linq;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Scrum.Web.Controllers;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;

namespace Scrum.Tests.Controllers
{
    [TestClass]
    public class ProjectControllerTest : BaseControllerTest
    {
        private ProjectController _controller;

        [TestInitialize]
        public void TestInitialize()
        {
            _controller = new ProjectController(_mockRepository.Object, _mockScrumMembershipProvider.Object, _mockScrumRoleProvider.Object);
            var mockedList = _controller.SetFakeControllerContextAndReturnMocksList("~/Project/");
        }

        //[TestMethod]
        //public void Verify_All_ActionResult_Method_Is_Decorated_With_Authorize_Attribute()
        //{
        //    const int expectedNumAuthorizeAttribute = 1;
        //    const string expectedRole = "admin";

        //    var type = _controller.GetType();
        //    var methods = type.GetMethods();
        //    foreach (var method in methods)
        //    {
        //        if (method.ReturnType == typeof(ActionResult))
        //        {
        //            var attributes = method.GetCustomAttributes(typeof(AuthorizeAttribute), true);
        //            Assert.AreEqual(expectedNumAuthorizeAttribute, attributes.Length, "El mètode {0} no té l'atribut 'Authorize'", method.Name);
        //            Assert.AreEqual(expectedRole, (attributes[0] as AuthorizeAttribute).Roles, "El mètode {0} no té definit el rol '{1}' per l'atribut 'Authorize'", method.Name, expectedRole);
        //        }
        //    }
        //}

        [TestMethod]
        public void ProjectControllerTest_Verify_ProjectController_Is_Decorated_With_Authorize_Attribute_And_Role_Admin()
        {
            const int expectedNumAuthorizeAttribute = 1;
            const string expectedRole = "admin";

            var type = _controller.GetType();

            var attributes = type.GetCustomAttributes(typeof(AuthorizeAttribute), true);
            Assert.AreEqual(expectedNumAuthorizeAttribute, attributes.Length, "El controlador {0} no té l'atribut 'Authorize'", type.Name);
            Assert.AreEqual(expectedRole, (attributes[0] as AuthorizeAttribute).Roles, "El controlador {0} no té definit el rol '{1}' per l'atribut 'Authorize'", type.Name, expectedRole);

        }

        [TestMethod]
        public void ProjectControllerTest_Index_Action_Returns_IEnumerable_ProjectListModel_To_The_View()
        {
            // Disponer
            ResetCalls();
            _mockRepository
                .Setup(ctx => ctx.All<Project>())
                .Returns(new List<Project>().AsQueryable());

            // Actuar
            var result = _controller.Index(null) as ViewResult;

            // Declarar
            var model = result.Model as IEnumerable<ProjectListModel>;
            Assert.IsNotNull(model, "El model ha de tornar un IEnumerable<ProjectListModel>");
        }

        [TestMethod]
        public void ProjectControllerTest_Add_Project_Action_Returns_RedirectToRouteResult_When_The_Customer_Model_Is_Valid()
        {
            // Disponer
            const string expectedRouteName = "Index";
            ProjectCreateModel model = new ProjectCreateModel();
            ResetCalls();

            // Actuar
            var result = _controller.Create(model) as RedirectToRouteResult;

            // Declarar
            Assert.IsNotNull(result, "Hauria de tornar un RedirectToRouteResult");
            Assert.AreEqual(expectedRouteName, result.RouteValues["action"], "La RouteName hauria de ser {0}", expectedRouteName);
        }

        [TestMethod]
        public void ProjectControllerTest_Add_Project_Action_Returns_RedirectToRouteResult_When_The_Customer_Model_Is_InValid()
        {
            // Disponer
            const string expectedViewName = "";
            ProjectCreateModel model = new ProjectCreateModel();
            ResetCalls();
            _controller.ModelState.AddModelError("Error", "Error");

            // Actuar
            var result = _controller.Create(model) as ViewResult;

            // Declarar
            Assert.IsNotNull(result, "Hauria de tornar un ViewResult");
            Assert.AreEqual(expectedViewName, result.ViewName, "La ViewName hauria de ser {0}", expectedViewName);
        }
        
    }
}
