﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;

namespace Scrum.Web.Controllers
{
    public class ProductBackLogController : Controller
    {
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public ProductBackLogController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        //
        // GET: /ProductBackLog/

        public ActionResult Index(int? projectId, int? page)
        {
            var selectedProjects = this._membershipService.GetSelectedProjectsAsProductOwner();
            var projects = this._repository.Query<Project>(proj => selectedProjects.Contains(proj.ProjectId));

            var model = Mapper.Map<IEnumerable<ProductBackLogListModel>>(projects);

            foreach (var item in model)
            {
                var pageNumber = 1;
                if (item.ProjectId == projectId)
                    pageNumber = page ?? 1;

                item.UserHistories = item.UserHistories.ToPagedList(pageNumber, Config.PageSize);
            }

            return View(model);
        }

        
        //
        // GET: /ProductBackLog/Create

        public ActionResult Create(int? projectId)
        {
            var project = this._repository.Single<Project>(projectId);
            var taskStatus = this._repository.All<UserHistoryStatus>().Where(uh => uh.UserHistoryStatusId == (int)Config.UserHistoryStatus.Pending);
            if (project == null)
            {
                return HttpNotFound();
            }

            var model = Mapper.Map<ProductBackLogCreateModel>(Tuple.Create(project, taskStatus));

            return View(model);
        }

        //
        // POST: /ProductBackLog/Create

        [HttpPost]
        public ActionResult Create(ProductBackLogCreateModel model)
        {
            if (ModelState.IsValid)
            {
                
                var userHistory = new UserHistory();
                userHistory.Name = model.Name;
                userHistory.Description = model.Description;
                userHistory.ProjectId = model.ProjectId;
                userHistory.UserHistoryStatusId = model.UserHistoryStatusId;
                userHistory.Priority = model.Priority;
                userHistory.Estimation = model.Estimation;

                this._repository.Add<UserHistory>(userHistory);
                this._repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /ProductBackLog/Edit/5

        public ActionResult Edit(long id = 0)
        {
            var userHistory = this._repository.Single<UserHistory>(id);

            var model = Mapper.Map<ProductBackLogEditModel>(userHistory);

            return View(model);
        }

        //
        // POST: /ProductBackLog/Edit/5

        [HttpPost]
        public ActionResult Edit(ProductBackLogEditModel model)
        {
            if (ModelState.IsValid)
            {
                var userHistory = this._repository.Single<UserHistory>(model.UserHistoryId);
                userHistory.Name = model.Name;
                userHistory.Description = model.Description;
                userHistory.Priority = model.Priority;
                userHistory.Estimation = model.Estimation;

                this._repository.SaveChanges();

                return RedirectToAction("Index");
            }

            return View(model);
        }

        //
        // GET: /ProductBackLog/Delete/5

        public ActionResult Delete(long id = 0)
        {
            this._repository.Delete<UserHistory>(id);
            this._repository.SaveChanges();
            return RedirectToAction("Index");
        }

        
        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }
    }
}