﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AutoMapper;
using Scrum.Web.Helpers;
using Scrum.Web.Models.DAL;
using Scrum.Web.Models.UI;
using PagedList;

namespace Scrum.Web.Controllers
{
    public class ProjectSelectorController : Controller
    {
        private readonly IRepository _repository;
        private readonly IScrumMembershipProvider _membershipService;
        private readonly IScrumRoleProvider _authorizationService;

        public ProjectSelectorController(IRepository repository, IScrumMembershipProvider membershipService, IScrumRoleProvider authorizationService)
        {
            this._repository = repository;
            this._membershipService = membershipService;
            this._authorizationService = authorizationService;
        }

        protected override void Dispose(bool disposing)
        {
            this._repository.Dispose();
            base.Dispose(disposing);
        }

        [ChildActionOnly]
        public PartialViewResult ProjectSelector()
        {
            var userId = (int)this._membershipService.GetUser().ProviderUserKey;
            var projects = this._repository.Query<Project>(pr => pr.Permission.Any(per => per.UserId == userId));

            var model = Mapper.Map<ProjectSelectorModel>(projects);

            model.SelectedProjects = this._membershipService.GetSelectedProjects();

            return PartialView(model);
        }

        
        public void Selected(ProjectSelectorModel model)
        {
            if (model != null)
            {
                if (model.SelectedProjects != null)
                {
                    Session["SelectedProjects"] = model.SelectedProjects;
                }
            }
        }
    }
}