﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using System.ComponentModel.DataAnnotations;

namespace System.Web.Mvc.Html
{
    public static class HTMLHelpers
    {
        public static MvcHtmlString ActionImage(this HtmlHelper html, string action, object routeValues, string imagePath, string alt)
        {
            var url = new UrlHelper(html.ViewContext.RequestContext);

            // build the <img> tag
            var imgBuilder = new TagBuilder("img");
            imgBuilder.MergeAttribute("src", url.Content(imagePath));
            imgBuilder.MergeAttribute("alt", alt);
            string imgHtml = imgBuilder.ToString(TagRenderMode.SelfClosing);

            // build the <a> tag
            var anchorBuilder = new TagBuilder("a");
            anchorBuilder.MergeAttribute("href", url.Action(action, routeValues));
            anchorBuilder.InnerHtml = imgHtml; // include the <img> tag inside
            string anchorHtml = anchorBuilder.ToString(TagRenderMode.Normal);

            return MvcHtmlString.Create(anchorHtml);
        }

        public static MvcHtmlString ActionImage(this HtmlHelper html, string action, string controllerName, string imagePath, string alt)
        {
            var url = new UrlHelper(html.ViewContext.RequestContext);

            // build the <img> tag
            var imgBuilder = new TagBuilder("img");
            imgBuilder.MergeAttribute("src", url.Content(imagePath));
            imgBuilder.MergeAttribute("alt", alt);
            string imgHtml = imgBuilder.ToString(TagRenderMode.SelfClosing);

            // build the <a> tag
            var anchorBuilder = new TagBuilder("a");
            anchorBuilder.MergeAttribute("href", url.Action(action, controllerName));
            anchorBuilder.InnerHtml = imgHtml; // include the <img> tag inside
            string anchorHtml = anchorBuilder.ToString(TagRenderMode.Normal);

            return MvcHtmlString.Create(anchorHtml);
        }

        public static MvcHtmlString ActionButton(this HtmlHelper html, string action, string controller, string text)
        {
            var url = new UrlHelper(html.ViewContext.RequestContext);

            var button = new TagBuilder("input");
            button.MergeAttribute("type", "button");
            button.MergeAttribute("value", text);
            button.MergeAttribute("onclick", string.Format("location.href='{0}'", url.Action(action, controller)));
            string buttonHtml = button.ToString(TagRenderMode.SelfClosing);

            return MvcHtmlString.Create(buttonHtml);
        }

        public static MvcHtmlString ActionButton(this HtmlHelper html, string action, string controller, string text, object routeValues)
        {
            var url = new UrlHelper(html.ViewContext.RequestContext);

            var button = new TagBuilder("input");
            button.MergeAttribute("type", "button");
            button.MergeAttribute("value", text);
            button.MergeAttribute("onclick", string.Format("location.href='{0}'", url.Action(action, controller, routeValues)));
            string buttonHtml = button.ToString(TagRenderMode.SelfClosing);

            return MvcHtmlString.Create(buttonHtml);
        }

        public static MvcHtmlString MyTextBoxFor<TModel, TValue>(this HtmlHelper<TModel> helper, Expression<Func<TModel, TValue>> expression)
        {
            var metaData = ModelMetadata.FromLambdaExpression(expression, helper.ViewData);
            
            var editable = metaData.ContainerType.GetProperty(metaData.PropertyName)
                            .GetCustomAttributes(typeof(EditableAttribute), false).SingleOrDefault() as EditableAttribute;

            if (editable == null || editable.AllowEdit)
                return helper.TextBoxFor(expression);
            else
                return helper.TextBoxFor(expression, new { @readonly = "readonly" });
        }

        public static MvcHtmlString MyEditorFor<TModel, TValue>(this HtmlHelper<TModel> helper, Expression<Func<TModel, TValue>> expression)
        {
            var metaData = ModelMetadata.FromLambdaExpression(expression, helper.ViewData);

            var editable = metaData.ContainerType.GetProperty(metaData.PropertyName)
                            .GetCustomAttributes(typeof(EditableAttribute), false).SingleOrDefault() as EditableAttribute;

            var stringLength = metaData.ContainerType.GetProperty(metaData.PropertyName)
                            .GetCustomAttributes(typeof(StringLengthAttribute), false).SingleOrDefault() as StringLengthAttribute;

            var maximumLength = 0;

            if (stringLength != null) maximumLength = stringLength.MaximumLength;

            var displayFormat = metaData.ContainerType.GetProperty(metaData.PropertyName)
                            .GetCustomAttributes(typeof(DisplayFormatAttribute), false).SingleOrDefault() as DisplayFormatAttribute;

            if (editable == null || editable.AllowEdit)
            {
                switch (Type.GetTypeCode(expression.ReturnType))
                {
                    case TypeCode.DateTime:
                        return helper.EditorFor(expression);

                    default:
                        if (maximumLength > 100)
                            return helper.TextAreaFor(expression);
                        else
                            return helper.EditorFor(expression);
                }
            }
            else
            {
                switch (Type.GetTypeCode(expression.ReturnType))
                {
                    case TypeCode.Boolean:
                        var expressionBool = Expression.Lambda<Func<TModel, bool>>(expression.Body, expression.Parameters);
                        return helper.CheckBoxFor(expressionBool, new { @disabled = "disabled" });

                    case TypeCode.DateTime:
                        if (displayFormat == null || string.IsNullOrWhiteSpace(displayFormat.DataFormatString))
                            return helper.TextBoxFor(expression, new { @readonly = "readonly" });
                        else
                            return helper.TextBoxFor(expression, displayFormat.DataFormatString, new { @readonly = "readonly" });

                    default:
                        if (maximumLength > 100)
                            return helper.TextAreaFor(expression, new { @readonly = "readonly" });
                        else
                            return helper.TextBoxFor(expression, new { @readonly = "readonly" });
                }
            }
        }

    }
}