﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Security;
using Scrum.Web.Models;
using Scrum.Web.Models.DAL;


namespace Scrum.Web.Helpers
{
    public class ScrumMembershipProvider : MembershipProvider, IScrumMembershipProvider
    {
        public override string ApplicationName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            throw new NotImplementedException();
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            ValidatePasswordEventArgs args = new ValidatePasswordEventArgs(username, password, true);

            OnValidatingPassword(args);

            if (args.Cancel)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            if ((RequiresUniqueEmail && (GetUserNameByEmail(email) != String.Empty)))
            {
                status = MembershipCreateStatus.DuplicateEmail;
                return null;
            }

            MembershipUser memberUser = GetUser(username, false);

            if (memberUser == null)
            {
                try
                {
                    using (ScrumEntities model = new ScrumEntities())
                    {
                        User user = new User();
                        user.Name = username;
                        user.Password = password;
                        user.Email = email;

                        model.User.Add(user);

                        model.SaveChanges();

                        status = MembershipCreateStatus.Success;

                        return GetUser(username, false);
                    }

                }
                catch
                {
                    status = MembershipCreateStatus.ProviderError;
                }
            }
            else
            {
                status = MembershipCreateStatus.DuplicateUserName;
            }

            return null;
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            throw new NotImplementedException();
        }

        public override bool EnablePasswordReset
        {
            get { throw new NotImplementedException(); }
        }

        public override bool EnablePasswordRetrieval
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new NotImplementedException();
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            MembershipUser membershipUser = null;

            using (ScrumEntities model = new ScrumEntities())
            {
                User user = (from u in model.User
                             where u.Name == username
                             select u)
                                 .FirstOrDefault();

                if (user != null)
                {
                    membershipUser = new MembershipUser(this.Name,
                        user.Name,
                        user.UserId,
                        user.Email,
                        string.Empty,
                        string.Empty,
                        true,
                        false,
                        DateTime.Now,
                        DateTime.Now,
                        DateTime.Now,
                        default(DateTime),
                        default(DateTime));
                }
            }

            return membershipUser;
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            throw new NotImplementedException();
        }

        public override string GetUserNameByEmail(string email)
        {
            using (ScrumEntities model = new ScrumEntities())
            {
                User user = (from u in model.User
                             where u.Email == email
                             select u)
                                 .FirstOrDefault();

                if (user == null)
                    return string.Empty;
                else
                    return user.Name;
            }
        }

        public override int MaxInvalidPasswordAttempts
        {
            get { throw new NotImplementedException(); }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { throw new NotImplementedException(); }
        }

        public override int MinRequiredPasswordLength
        {
            get { throw new NotImplementedException(); }
        }

        public override int PasswordAttemptWindow
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { return MembershipPasswordFormat.Clear; }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresUniqueEmail
        {
            get { return true; }
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotImplementedException();
        }

        public override void UpdateUser(MembershipUser user)
        {
            throw new NotImplementedException();
        }

        public override bool ValidateUser(string username, string password)
        {
            bool isValid = false;

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    User user = (from u in model.User
                                 where u.Name == username
                                 select u).FirstOrDefault();

                    if (user != null)
                    {
                        string storedPassword = user.Password;
                        if (CheckPassword(password, storedPassword))
                        {
                            isValid = true;
                        }
                    }
                }
                catch
                {
                    isValid = false;
                }
            }
            return isValid;
        }

        private bool CheckPassword(string password, string dbpassword)
        {
            string pass1 = password;
            string pass2 = dbpassword;

            switch (PasswordFormat)
            {
                case MembershipPasswordFormat.Encrypted:
                    throw new NotImplementedException();

                case MembershipPasswordFormat.Hashed:
                    throw new NotImplementedException();

                default:
                    break;
            }

            if (pass1 == pass2)
            {
                return true;
            }

            return false;
        }

        public MembershipUser GetUser()
        {
            return Membership.GetUser();
        }

        public List<int> GetSelectedProjects()
        {
            if (HttpContext.Current.Session["SelectedProjects"] == null)
                InitializeSelectedProjects();

            return HttpContext.Current.Session["SelectedProjects"] as List<int>;
        }

        private void InitializeSelectedProjects()
        {
            var userId = (int)GetUser().ProviderUserKey;
            using (ScrumEntities model = new ScrumEntities())
            {
                var projects =  model.Project.Where(pr => pr.Permission.Any(per => per.UserId == userId));
                HttpContext.Current.Session["SelectedProjects"] = new List<int>(projects.Select(pr => pr.ProjectId));
            }
        }

        public List<int> GetSelectedProjectsAsProductOwner()
        {
            using (ScrumEntities model = new ScrumEntities())
            {
                var selectedProjects = GetSelectedProjects();
                var userId = (int)GetUser().ProviderUserKey;
                var result = model.Permission.Where(per => 
                        (per.ProfileId == (int)Config.Profile.ProductOwner || per.ProfileId == (int)Config.Profile.ScrumMaster)
                                               && per.UserId == userId
                                               && selectedProjects.Contains(per.ProjectId))
                                .Select(per => per.ProjectId).ToList();

                return result;
            }
        }
    }
}