﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Security;
using Scrum.Web.Models.DAL;



namespace Scrum.Web.Helpers
{
    public class ScrumRoleProvider : RoleProvider, IScrumRoleProvider
    {
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override string ApplicationName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override void CreateRole(string roleName)
        {
            using (ScrumEntities model = new ScrumEntities())
            {
                Profile role = new Profile();
                role.Name = roleName;

                model.Profile.Add(role);

                model.SaveChanges();
            }
        }

        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            bool result = false;

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    var role = (from r in model.Profile
                                where r.Name == roleName
                                select r).SingleOrDefault();

                    if (role != null)
                    {
                        model.Profile.Remove(role);

                        model.SaveChanges();

                        result = true;
                    }
                }
                catch
                {
                    result = false;
                }
            }

            return result;
        }

        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            List<string> users = new List<string>();

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    if (roleName == "admin")
                    {
                        var usersAdmin = from ua in model.User
                                         where ua.IsAdmin == 1
                                         select ua;

                        if (usersAdmin != null)
                        {
                            foreach (var userAdmin in usersAdmin)
                            {
                                users.Add(userAdmin.Name);
                            }
                        }

                    }
                    else
                    {
                        var usersInRole = (from uir in model.Permission
                                          where uir.Profile.Name == roleName && uir.User.Name == usernameToMatch
                                          select uir.User.Name).Distinct();

                        if (usersInRole != null)
                        {
                            foreach (var userInRole in usersInRole)
                            {
                                users.Add(userInRole);
                            }
                        }
                    }
                }
                catch
                {
                }
            }

            return users.ToArray();
        }

        public override string[] GetAllRoles()
        {
            List<string> roles = new List<string>();

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    var dbRoles = from r in model.Profile
                                  select r;

                    foreach (var role in dbRoles)
                    {
                        roles.Add(role.Name);
                    }

                    roles.Add("admin");
                }
                catch
                {
                }
            }

            return roles.ToArray();
        }

        public override string[] GetRolesForUser(string username)
        {
            List<string> roles = new List<string>();

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    // Get Roles from projects
                    var dbRoles = (from r in model.Permission
                                  where r.User.Name == username
                                  select r.Profile.Name).Distinct();

                    foreach (var role in dbRoles)
                    {
                        roles.Add(role);
                    }

                    // Get roles from admins
                    var user = (from us in model.User
                                   where us.Name == username
                                   select us).FirstOrDefault();

                    if (user != null)
                    {
                        if (user.IsAdmin == 1)
                        {
                            roles.Add("admin");
                        }
                    }
                   

                }
                catch
                {
                }
            }

            return roles.ToArray();
        }

        public override string[] GetUsersInRole(string roleName)
        {
            List<string> users = new List<string>();

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    // Permission by project
                    var usersInRole = (from uir in model.Permission
                                      where uir.Profile.Name == roleName
                                      select uir.User.Name).Distinct();

                    if (usersInRole != null)
                    {
                        foreach (var userInRole in usersInRole)
                        {
                            users.Add(userInRole);
                        }
                    }

                    // Admin permission
                    var usersAdmin = (from ua in model.User
                                      where ua.IsAdmin == 1
                                      select ua.Name);

                    if (usersAdmin != null)
                    {
                        foreach (var userAdmin in usersAdmin)
                        {
                            users.Add(userAdmin);
                        }
                    }
                }
                catch
                {
                }
            }

            return users.ToArray();
        }

        public override bool IsUserInRole(string username, string roleName)
        {
            bool result = false;

            using (ScrumEntities model = new ScrumEntities())
            {
                try
                {
                    var usersInRole = from uir in model.Permission
                                      where uir.User.Name == username && uir.Profile.Name == roleName
                                      select uir;

                    if (usersInRole != null)
                    {
                        result = true;
                    }
                }
                catch
                {
                    result = false;
                }
            }

            return result;
        }

        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override bool RoleExists(string roleName)
        {
            bool exist = false;

            using (ScrumEntities model = new ScrumEntities())
            {
                if (model.Profile.Any(r => r.Name == roleName))
                {
                    exist = true;
                }
            }

            return exist;
        }
    }
}