﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace Scrum.Web.Models.DAL
{
    public interface IRepository : IDisposable
    {
        void Add<TModel>(TModel instance) where TModel : class, IEntity;
        void Add<TModel>(IEnumerable<TModel> instances) where TModel : class, IEntity;

        IQueryable<TModel> All<TModel>() where TModel : class, IEntity;

        void Delete<TModel>(object key) where TModel : class, IEntity;
        void Delete<TModel>(TModel instance) where TModel : class, IEntity;
        void Delete<TModel>(Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity;
        void DeleteAll<TModel>(Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity;

        TModel Single<TModel>(object key) where TModel : class, IEntity;
        TModel Single<TModel>(Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity;

        IQueryable<TModel> Query<TModel>(Expression<Func<TModel, bool>> predicate) where TModel : class, IEntity;

        int SaveChanges();
    }
}
