﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;

namespace Scrum.Web.Models.UI
{
    public class SprintListModel
    {
        [Display(Name = "Identificador")]
        public int ProjectId { get; set; }
        
        [Display(Name = "Nom curt")]
        public string ShortName { get; set; }

        [Display(Name = "Nom")]
        public string Name { get; set; }

        public long SprintId { get; set; }

        public bool CanCreateSprint { get; set; }

        public bool CanFinnishSprint { get; set; }

        public bool CanAssignOrFinnishTask { get; set; }

        public DateTime? BeginDate { get; set; }

        public DateTime? FinnishDate { get; set; }
        
        [Display(Name = "Tasques")]
        public IEnumerable<SprintTaskModel> Tasks { get; set; }
    }

    public class SprintUserHistoryModel
    {
        public long UserHistoryId { get; set; }
        public int UserHistoryStatusId { get; set; }

        [Display(Name = "Prioritat")]
        public int Priority { get; set; }
        
        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Display(Name = "Descripció")]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació història usuari")]
        public decimal Estimation { get; set; }

        public int ProjectId { get; set; }

        [Display(Name = "Entra al esprint?")]
        public bool Selected { get; set; }

        [Display(Name = "Estimació tasques")]
        public decimal TotalTaskEstimation { get; set; }
    }

    public class SprintTaskModel
    {
        public long TaskId { get; set; }

        [Display(Name = "Codi història usuari")]
        public long UserHistoryId { get; set; }
        
        [Display(Name = "Estat")]
        public int TaskStatusId { get; set; }

        [Display(Name = "Estat")]
        public string TaskStatus { get; set; }


        [Display(Name = "Assignació")]
        public Nullable<int> AssignedUserId { get; set; }

        [Display(Name = "Assignat")]
        public string AssignedUserName { get; set; }
        
        [Display(Name = "Nom")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        public string Name { get; set; }
        
        [Display(Name = "Descripció")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació")]
        public decimal Estimation { get; set; }

        [Display(Name = "Temps invertit")]
        public decimal TotalWorkLog { get; set; }

        [Display(Name = "Història d'usuari")]
        [Editable(false)]
        public string UserHistoryName { get; set; }

        public int ProjectId { get; set; }

        public List<SprintUserModel> Developers { get; set; }
    }

    public class SprintAssignTaskModel
    {
        public long TaskId { get; set; }
        public long UserHistoryId { get; set; }

        [Display(Name = "Estat")]
        public int TaskStatusId { get; set; }

        [Display(Name = "Assignació")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        public Nullable<int> AssignedUserId { get; set; }

        [Display(Name = "Nom")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        public string Name { get; set; }

        [Display(Name = "Descripció")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació")]
        [Editable(false)]
        public decimal Estimation { get; set; }

        [Display(Name = "Història d'usuari")]
        [Editable(false)]
        public string UserHistoryName { get; set; }

        public int ProjectId { get; set; }

        public long SprintId { get; set; }

        public List<SprintUserModel> Developers { get; set; }
    }

    public class SprintFinnishTaskModel
    {
        public long TaskId { get; set; }
        public long UserHistoryId { get; set; }

        [Display(Name = "Estat")]
        public int TaskStatusId { get; set; }

        [Display(Name = "Assignació")]
        public Nullable<int> AssignedUserId { get; set; }

        [Display(Name = "Assignació")]
        [Editable(false)]
        public string AssignedUserName { get; set; }

        [Display(Name = "Nom")]
        [Editable(false)]
        public string Name { get; set; }

        [Display(Name = "Descripció")]
        [Editable(false)]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació")]
        [Editable(false)]
        public decimal Estimation { get; set; }

        [Display(Name = "Història d'usuari")]
        [Editable(false)]
        public string UserHistoryName { get; set; }

        [Display(Name = "Temps invertit")]
        public decimal WorkLogItem { get; set; }

        public int ProjectId { get; set; }

        public long SprintId { get; set; }
    }

    public class SprintUserModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        [Editable(false)]
        public string Name { get; set; }
    }

    public class SprintCreateModel
    {
        public int ProjectId { get; set; }
        
        [Display(Name="Nom projecte")]
        [Editable(false)]
        public string Name { get; set; }

        [Display(Name = "Data inici")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [DataType(DataType.DateTime)]
        public DateTime? BeginDate { get; set; }

        [Display(Name = "Data fi")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [DataType(DataType.DateTime)]
        public DateTime? FinnishDate { get; set; }

        [Display(Name = "Històries d'usuari")]
        public List<SprintUserHistoryModel> UserHistories { get; set; }
    }

    public class SprintEditModel
    {
        public long UserHistoryId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Estat")]
        [Editable(false)]
        public int UserHistoryStatusId { get; set; }

        [Display(Name = "Prioritat")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        public int Priority { get; set; }

        [Display(Name = "Nom")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        public string Name { get; set; }

        [Display(Name = "Descripció")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació")]
        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Editable(false)]
        public decimal Estimation { get; set; }

        public int ProjectId { get; set; }

        [Display(Name = "Nom projecte")]
        [Editable(false)]
        public string ProjectName { get; set; }

        [Display(Name = "Estat")]
        [Editable(false)]
        public string UserHistoryStatusDescription { get; set; }

        public List<SprintTaskModel> Tasks { get; set; }
    }
}