﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using DotNet.Highcharts;

namespace Scrum.Web.Models.UI
{
    public class StatsListModel
    {
        [Display(Name = "Identificador")]
        public int ProjectId { get; set; }
        
        [Display(Name = "Nom curt")]
        public string ShortName { get; set; }

        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Display(Name = "Esprints")]
        public IEnumerable<StatsSprintModel> Sprints { get; set; }
    }

    public class StatsSprintModel
    {
        [Display(Name = "Codi esprint")]
        public long SprintId { get; set; }
        
        public int ProjectId { get; set; }


        [Display(Name = "Projecte")]
        [Editable(false)]
        public string ProjectName { get; set; }

        [Display(Name = "Data inici esprint")]
        [DisplayFormat(ApplyFormatInEditMode=true, DataFormatString = "{0:dd/MM/yyyy}")]
        [Editable(false)]
        public System.DateTime BeginDate { get; set; }

        [Display(Name = "Data fi esprint")]
        [DisplayFormat(DataFormatString = "{0:dd/MM/yyyy}")]
        [Editable(false)]
        public System.DateTime FinnishDate { get; set; }

        [Display(Name = "Durada")]
        [DisplayFormat(DataFormatString = "{0:dd/MM/yyyy}")]
        public string ElapsedDays
        {
            get
            {
                return string.Format("{0} dies", NumberOfElapsedDays);
            }
        }

        public int NumberOfElapsedDays
        {
            get
            {
                var dias = FinnishDate - BeginDate;
                return Convert.ToInt32(dias.TotalDays);
            }
        }

        public IEnumerable<StatsUserHistoryModel> UserHistories { get; set; }

        public Highcharts Chart { get; set; }
    }

    public class StatsUserHistoryModel
    {
        public long UserHistoryId { get; set; }
        public int UserHistoryStatusId { get; set; }

        [Display(Name = "Prioritat")]
        public int Priority { get; set; }

        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Display(Name = "Descripció")]
        [StringLength(2000)]
        public string Description { get; set; }

        [Display(Name = "Estimació història usuari")]
        public decimal Estimation { get; set; }

        public int ProjectId { get; set; }

        [Display(Name = "Estimació tasques")]
        public decimal TotalTaskEstimation { get; set; }

        [Display(Name = "Número de tasques")]
        public int NumberOfTasks { get; set; }
    }

    
}