﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;

namespace Scrum.Web.Models.UI
{
    public class UserListModel
    {
        [Display(Name = "Identificador")]
        public int UserId { get; set; }

        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Display(Name = "Correu electrònic")]
        public string Email { get; set; }

        [Display(Name = "Administrador")]
        public bool IsAdmin { get; set; }
    }

    public class UserCreateModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Contrasenya")]
        [StringLength(100, ErrorMessage = "El número de caràcters de {0} ha de ser com a mínim de {2}.", MinimumLength = 6)]
        [DataType(DataType.Password)]
        public string Password { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Confirmar contrasenya")]
        [DataType(DataType.Password)]
        [Compare("Password", ErrorMessage = "La  contrasenya i la contrasenya de confirmació no coincideixen.")]
        public string ConfirmPassword { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Correu electrònic")]
        public string Email { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Administrador")]
        public bool IsAdmin { get; set; }
    }

    public class UserEditModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Correu electrònic")]
        public string Email { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Administrador")]
        public bool IsAdmin { get; set; }
    }

    public class UserDetailModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        [Editable(false)]
        public string Name { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Correu electrònic")]
        [Editable(false)]
        public string Email { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Administrador")]
        [Editable(false)]
        public bool IsAdmin { get; set; }
    }

    public class UserNewPasswordModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        [Editable(false)]
        public string Name { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Contrasenya")]
        [StringLength(100, ErrorMessage = "El número de caràcters de {0} ha de ser com a mínim de {2}.", MinimumLength = 6)]
        [DataType(DataType.Password)]
        public string Password { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Confirmar contrasenya")]
        [DataType(DataType.Password)]
        [Compare("Password", ErrorMessage = "La  contrasenya i la contrasenya de confirmació no coincideixen.")]
        public string ConfirmPassword { get; set; }

    }

    public class UserRoleListModel
    {
        [Display(Name = "Id")]
        [Editable(false)]
        public int? UserId { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Nom")]
        [Editable(false)]
        public string Name { get; set; }

        [Required(ErrorMessage="El camp {0} és obligatori.")]
        [Display(Name = "Correu electrònic")]
        [Editable(false)]
        public string Email { get; set; }

        public List<UserRoleListModelProject> Owners { get; set; }
        public List<UserRoleListModelProject> ScrumMasters { get; set; }
        public List<UserRoleListModelProject> Developers { get; set; }
    }

    public class UserRoleListModelProject
    {
        [Display(Name = "Identificador")]
        public int ProjectId { get; set; }

        [Display(Name = "Nom curt")]
        public string ShortName { get; set; }

        [Display(Name = "Nom")]
        public string Name { get; set; }

        [Display(Name = "Identificador perfil")]
        public int ProfileId { get; set; }

        [Display(Name = "Permís")]
        public bool HasAccess { get; set; }

    }
}